@@ -13,7 +13,7 @@ set_or_print_default_version()
     echo "This script can build the following packages:"
   fi
   # Get package name from argument passed with -p, -V, -D, or -U
-  package_name="${arg_p:-${arg_D:-${arg_P:-${arg_U:-${arg_V}}}}}" # not needed for -l
+  package_name="${arg_p:-${arg_D:-${arg_P:-${arg_U:-${arg_V:-${arg_B}}}}}}" # not needed for -l
 
   if [[ "${package_name}" == "ofp" ]]; then
     "${OPENCOARRAYS_SRC_DIR}/prerequisites/install-ofp.sh" "${@}"
@@ -26,8 +26,8 @@ set_or_print_default_version()
   # denominator because, for licensing reasons, OS X only has bash 3 by default.)
   # See http://stackoverflow.com/questions/1494178/how-to-define-hash-tables-in-bash
   package_version=(
-    "cmake:3.4.0"
-    "gcc:6.1.0"
+    "cmake:3.10.0"
+    "gcc:8.2.0"
     "mpich:3.2"
     "wget:1.16.3"
     "flex:2.6.0"
